@@ -26,11 +26,6 @@
 #ifndef PSYCOPG_DATETIME_H
 #define PSYCOPG_DATETIME_H 1
 
-#define PY_SSIZE_T_CLEAN
-#include <Python.h>
-
-#include "psycopg/config.h"
-
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -50,6 +45,8 @@ typedef struct {
 } pydatetimeObject;
 
 
+HIDDEN int psyco_adapter_datetime_init(void);
+
 /* functions exported to psycopgmodule.c */
 #ifdef PSYCOPG_DEFAULT_PYDATETIME
 
