@@ -23,38 +23,70 @@
  * License for more details.
  */
 
-#define PY_SSIZE_T_CLEAN
-#include <Python.h>
-#include <structmember.h>
-#include <floatobject.h>
-#include <math.h>
-
 #define PSYCOPG_MODULE
-#include "psycopg/config.h"
-#include "psycopg/python.h"
 #include "psycopg/psycopg.h"
+
 #include "psycopg/adapter_pfloat.h"
 #include "psycopg/microprotocols_proto.h"
 
+#include <floatobject.h>
+#include <math.h>
+
 
 /** the Float object **/
 
 static PyObject *
-pfloat_str(pfloatObject *self)
+pfloat_getquoted(pfloatObject *self, PyObject *args)
 {
+    PyObject *rv;
     double n = PyFloat_AsDouble(self->wrapped);
     if (isnan(n))
-        return PyString_FromString("'NaN'::float");
-    else if (isinf(n))
-        return PyString_FromString("'Infinity'::float");
-    else
-        return PyObject_Repr(self->wrapped);
+        rv = Bytes_FromString("'NaN'::float");
+    else if (isinf(n)) {
+        if (n > 0)
+            rv = Bytes_FromString("'Infinity'::float");
+        else
+            rv = Bytes_FromString("'-Infinity'::float");
+    }
+    else {
+        if (!(rv = PyObject_Repr(self->wrapped))) {
+            goto exit;
+        }
+
+#if PY_MAJOR_VERSION > 2
+        /* unicode to bytes in Py3 */
+        {
+            PyObject *tmp = PyUnicode_AsUTF8String(rv);
+            Py_DECREF(rv);
+            if (!(rv = tmp)) {
+                goto exit;
+            }
+        }
+#endif
+
+        if ('-' == Bytes_AS_STRING(rv)[0]) {
+            /* Prepend a space in front of negative numbers (ticket #57) */
+            PyObject *tmp;
+            if (!(tmp = Bytes_FromString(" "))) {
+                Py_DECREF(rv);
+                rv = NULL;
+                goto exit;
+            }
+            Bytes_ConcatAndDel(&tmp, rv);
+            if (!(rv = tmp)) {
+                goto exit;
+            }
+        }
+    }
+
+exit:
+    return rv;
 }
 
 static PyObject *
-pfloat_getquoted(pfloatObject *self, PyObject *args)
+pfloat_str(pfloatObject *self)
 {
-    return pfloat_str(self);
+    return psycopg_ensure_text(pfloat_getquoted(self, NULL));
 }
 
 static PyObject *
@@ -78,7 +110,7 @@ pfloat_conform(pfloatObject *self, PyObject *args)
 /* object member list */
 
 static struct PyMemberDef pfloatObject_members[] = {
-    {"adapted", T_OBJECT, offsetof(pfloatObject, wrapped), RO},
+    {"adapted", T_OBJECT, offsetof(pfloatObject, wrapped), READONLY},
     {NULL}
 };
 
@@ -98,7 +130,7 @@ pfloat_setup(pfloatObject *self, PyObject *obj)
 {
     Dprintf("pfloat_setup: init pfloat object at %p, refcnt = "
         FORMAT_CODE_PY_SSIZE_T,
-        self, ((PyObject *)self)->ob_refcnt
+        self, Py_REFCNT(self)
       );
 
     Py_INCREF(obj);
@@ -106,20 +138,11 @@ pfloat_setup(pfloatObject *self, PyObject *obj)
 
     Dprintf("pfloat_setup: good pfloat object at %p, refcnt = "
         FORMAT_CODE_PY_SSIZE_T,
-        self, ((PyObject *)self)->ob_refcnt
+        self, Py_REFCNT(self)
       );
     return 0;
 }
 
-static int
-pfloat_traverse(PyObject *obj, visitproc visit, void *arg)
-{
-    pfloatObject *self = (pfloatObject *)obj;
-
-    Py_VISIT(self->wrapped);
-    return 0;
-}
-
 static void
 pfloat_dealloc(PyObject* obj)
 {
@@ -129,10 +152,10 @@ pfloat_dealloc(PyObject* obj)
 
     Dprintf("pfloat_dealloc: deleted pfloat object at %p, refcnt = "
         FORMAT_CODE_PY_SSIZE_T,
-        obj, obj->ob_refcnt
+        obj, Py_REFCNT(obj)
       );
 
-    obj->ob_type->tp_free(obj);
+    Py_TYPE(obj)->tp_free(obj);
 }
 
 static int
@@ -152,12 +175,6 @@ pfloat_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
     return type->tp_alloc(type, 0);
 }
 
-static void
-pfloat_del(PyObject* self)
-{
-    PyObject_GC_Del(self);
-}
-
 static PyObject *
 pfloat_repr(pfloatObject *self)
 {
@@ -172,66 +189,43 @@ pfloat_repr(pfloatObject *self)
 "Float(str) -> new Float adapter object"
 
 PyTypeObject pfloatType = {
-    PyObject_HEAD_INIT(NULL)
-    0,
+    PyVarObject_HEAD_INIT(NULL, 0)
     "psycopg2._psycopg.Float",
-    sizeof(pfloatObject),
-    0,
+    sizeof(pfloatObject), 0,
     pfloat_dealloc, /*tp_dealloc*/
     0,          /*tp_print*/
-
     0,          /*tp_getattr*/
     0,          /*tp_setattr*/
-
     0,          /*tp_compare*/
-
     (reprfunc)pfloat_repr, /*tp_repr*/
     0,          /*tp_as_number*/
     0,          /*tp_as_sequence*/
     0,          /*tp_as_mapping*/
     0,          /*tp_hash */
-
     0,          /*tp_call*/
     (reprfunc)pfloat_str, /*tp_str*/
-
     0,          /*tp_getattro*/
     0,          /*tp_setattro*/
     0,          /*tp_as_buffer*/
-
-    Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
+    Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE, /*tp_flags*/
     pfloatType_doc, /*tp_doc*/
-
-    pfloat_traverse, /*tp_traverse*/
+    0,          /*tp_traverse*/
     0,          /*tp_clear*/
-
     0,          /*tp_richcompare*/
     0,          /*tp_weaklistoffset*/
-
     0,          /*tp_iter*/
     0,          /*tp_iternext*/
-
-    /* Attribute descriptor and subclassing stuff */
-
     pfloatObject_methods, /*tp_methods*/
     pfloatObject_members, /*tp_members*/
     0,          /*tp_getset*/
     0,          /*tp_base*/
     0,          /*tp_dict*/
-
     0,          /*tp_descr_get*/
     0,          /*tp_descr_set*/
     0,          /*tp_dictoffset*/
-
     pfloat_init, /*tp_init*/
-    0, /*tp_alloc  will be set to PyType_GenericAlloc in module init*/
+    0,          /*tp_alloc*/
     pfloat_new, /*tp_new*/
-    (freefunc)pfloat_del, /*tp_free  Low-level free-memory routine */
-    0,          /*tp_is_gc For PyObject_IS_GC */
-    0,          /*tp_bases*/
-    0,          /*tp_mro method resolution order */
-    0,          /*tp_cache*/
-    0,          /*tp_subclasses*/
-    0           /*tp_weaklist*/
 };
 
 
