@@ -25,7 +25,7 @@
 
 /* IMPORTANT NOTE: no function in this file do its own connection locking
    except for pg_execute and pq_fetch (that are somehow high-level). This means
-   that all the othe functions should be called while holding a lock to the
+   that all the other functions should be called while holding a lock to the
    connection.
 */
 
@@ -38,6 +38,7 @@
 #include "psycopg/green.h"
 #include "psycopg/typecast.h"
 #include "psycopg/pgtypes.h"
+#include "psycopg/error.h"
 
 #include <string.h>
 
@@ -64,7 +65,7 @@ strip_severity(const char *msg)
    code.  A list of error codes can be found at:
 
    http://www.postgresql.org/docs/current/static/errcodes-appendix.html */
-static PyObject *
+BORROWED static PyObject *
 exception_from_sqlstate(const char *sqlstate)
 {
     switch (sqlstate[0]) {
@@ -149,35 +150,40 @@ exception_from_sqlstate(const char *sqlstate)
 
 /* pq_raise - raise a python exception of the right kind
 
-   This function should be called while holding the GIL. */
+   This function should be called while holding the GIL.
 
-static void
-pq_raise(connectionObject *conn, cursorObject *curs, PGresult *pgres)
+   The function passes the ownership of the pgres to the returned exception,
+   where the pgres was the explicit argument or taken from the cursor.
+   So, after calling it curs->pgres will be set to null */
+
+RAISES static void
+pq_raise(connectionObject *conn, cursorObject *curs, PGresult **pgres)
 {
     PyObject *exc = NULL;
     const char *err = NULL;
     const char *err2 = NULL;
     const char *code = NULL;
+    PyObject *pyerr = NULL;
 
     if (conn == NULL) {
         PyErr_SetString(DatabaseError,
             "psycopg went psycotic and raised a null error");
         return;
     }
-    
+
     /* if the connection has somehow beed broken, we mark the connection
        object as closed but requiring cleanup */
     if (conn->pgconn != NULL && PQstatus(conn->pgconn) == CONNECTION_BAD)
         conn->closed = 2;
 
     if (pgres == NULL && curs != NULL)
-        pgres = curs->pgres;
+        pgres = &curs->pgres;
 
-    if (pgres) {
-        err = PQresultErrorMessage(pgres);
+    if (pgres && *pgres) {
+        err = PQresultErrorMessage(*pgres);
         if (err != NULL) {
             Dprintf("pq_raise: PQresultErrorMessage: err=%s", err);
-            code = PQresultErrorField(pgres, PG_DIAG_SQLSTATE);
+            code = PQresultErrorField(*pgres, PG_DIAG_SQLSTATE);
         }
     }
     if (err == NULL) {
@@ -210,7 +216,26 @@ pq_raise(connectionObject *conn, cursorObject *curs, PGresult *pgres)
     err2 = strip_severity(err);
     Dprintf("pq_raise: err2=%s", err2);
 
-    psyco_set_error(exc, curs, err2, err, code);
+    pyerr = psyco_set_error(exc, curs, err2);
+
+    if (pyerr && PyObject_TypeCheck(pyerr, &errorType)) {
+        errorObject *perr = (errorObject *)pyerr;
+
+        PyMem_Free(perr->codec);
+        psycopg_strdup(&perr->codec, conn->codec, 0);
+
+        Py_CLEAR(perr->pgerror);
+        perr->pgerror = error_text_from_chars(perr, err);
+
+        Py_CLEAR(perr->pgcode);
+        perr->pgcode = error_text_from_chars(perr, code);
+
+        CLEARPGRES(perr->pgres);
+        if (pgres && *pgres) {
+            perr->pgres = *pgres;
+            *pgres = NULL;
+        }
+    }
 }
 
 /* pq_set_critical, pq_resolve_critical - manage critical errors
@@ -249,7 +274,9 @@ pq_clear_critical(connectionObject *conn)
     }
 }
 
-static PyObject *
+/* return -1 if the exception is set (i.e. if conn->critical is set),
+ * else 0 */
+RAISES_NEG static int
 pq_resolve_critical(connectionObject *conn, int close)
 {
     Dprintf("pq_resolve_critical: resolving %s", conn->critical);
@@ -264,11 +291,13 @@ pq_resolve_critical(connectionObject *conn, int close)
 
         /* we don't want to destroy this connection but just close it */
         if (close == 1) conn_close(conn);
-    
+
         /* remember to clear the critical! */
-        pq_clear_critical(conn);    
+        pq_clear_critical(conn);
+
+        return -1;
     }
-    return NULL;
+    return 0;
 }
 
 /* pq_clear_async - clear the effects of a previous async query
@@ -300,19 +329,16 @@ pq_clear_async(connectionObject *conn)
 
    Accepted arg values are 1 (nonblocking) and 0 (blocking).
 
-   Return 0 if everything ok, else nonzero.
-
-   In case of error, if pyerr is nonzero, set a Python exception.
+   Return 0 if everything ok, else < 0 and set an exception.
  */
-int
-pq_set_non_blocking(connectionObject *conn, int arg, int pyerr)
+RAISES_NEG int
+pq_set_non_blocking(connectionObject *conn, int arg)
 {
     int ret = PQsetnonblocking(conn->pgconn, arg);
     if (0 != ret) {
         Dprintf("PQsetnonblocking(%d) FAILED", arg);
-        if (pyerr) {
-            PyErr_SetString(OperationalError, "PQsetnonblocking() failed");
-        }
+        PyErr_SetString(OperationalError, "PQsetnonblocking() failed");
+        ret = -1;
     }
     return ret;
 }
@@ -368,7 +394,7 @@ pq_execute_command_locked(connectionObject *conn, const char *query,
     }
 
     retvalue = 0;
-    IFCLEARPGRES(*pgres);
+    CLEARPGRES(*pgres);
 
 cleanup:
     return retvalue;
@@ -382,19 +408,31 @@ cleanup:
    This function should be called while holding the global interpreter
    lock.
  */
-void
+RAISES void
 pq_complete_error(connectionObject *conn, PGresult **pgres, char **error)
 {
     Dprintf("pq_complete_error: pgconn = %p, pgres = %p, error = %s",
             conn->pgconn, *pgres, *error ? *error : "(null)");
-    if (*pgres != NULL)
-        pq_raise(conn, NULL, *pgres);
-    else if (*error != NULL) {
-        PyErr_SetString(OperationalError, *error);
-    } else {
-        PyErr_SetString(OperationalError, "unknown error");
+    if (*pgres != NULL) {
+        pq_raise(conn, NULL, pgres);
+        /* now *pgres is null */
     }
-    IFCLEARPGRES(*pgres);
+    else {
+        if (*error != NULL) {
+            PyErr_SetString(OperationalError, *error);
+        } else {
+            PyErr_SetString(OperationalError, "unknown error");
+        }
+        /* Trivia: with a broken socket connection PQexec returns NULL, so we
+         * end up here. With a TCP connection we get a pgres with an error
+         * instead, and the connection gets closed in the pq_raise call above
+         * (see ticket #196)
+         */
+        if (CONNECTION_BAD == PQstatus(conn->pgconn)) {
+            conn->closed = 2;
+        }
+    }
+
     if (*error) {
         free(*error);
         *error = NULL;
@@ -444,38 +482,39 @@ pq_commit(connectionObject *conn)
     PGresult *pgres = NULL;
     char *error = NULL;
 
+    Py_BEGIN_ALLOW_THREADS;
+    pthread_mutex_lock(&conn->lock);
+
     Dprintf("pq_commit: pgconn = %p, autocommit = %d, status = %d",
             conn->pgconn, conn->autocommit, conn->status);
 
     if (conn->autocommit || conn->status != CONN_STATUS_BEGIN) {
         Dprintf("pq_commit: no transaction to commit");
-        return 0;
+        retvalue = 0;
+    }
+    else {
+        conn->mark += 1;
+        retvalue = pq_execute_command_locked(conn, "COMMIT", &pgres, &error, &_save);
     }
-
-    Py_BEGIN_ALLOW_THREADS;
-    pthread_mutex_lock(&conn->lock);
-    conn->mark += 1;
-
-    retvalue = pq_execute_command_locked(conn, "COMMIT", &pgres, &error, &_save);
 
     Py_BLOCK_THREADS;
     conn_notice_process(conn);
     Py_UNBLOCK_THREADS;
 
+    /* Even if an error occurred, the connection will be rolled back,
+       so we unconditionally set the connection status here. */
+    conn->status = CONN_STATUS_READY;
+
     pthread_mutex_unlock(&conn->lock);
     Py_END_ALLOW_THREADS;
 
     if (retvalue < 0)
         pq_complete_error(conn, &pgres, &error);
 
-    /* Even if an error occurred, the connection will be rolled back,
-       so we unconditionally set the connection status here. */
-    conn->status = CONN_STATUS_READY;
-
     return retvalue;
 }
 
-int
+RAISES_NEG int
 pq_abort_locked(connectionObject *conn, PGresult **pgres, char **error,
                 PyThreadState **tstate)
 {
@@ -502,7 +541,7 @@ pq_abort_locked(connectionObject *conn, PGresult **pgres, char **error,
    This function should be called while holding the global interpreter
    lock. */
 
-int
+RAISES_NEG int
 pq_abort(connectionObject *conn)
 {
     int retvalue = -1;
@@ -512,11 +551,6 @@ pq_abort(connectionObject *conn)
     Dprintf("pq_abort: pgconn = %p, autocommit = %d, status = %d",
             conn->pgconn, conn->autocommit, conn->status);
 
-    if (conn->autocommit || conn->status != CONN_STATUS_BEGIN) {
-        Dprintf("pq_abort: no transaction to abort");
-        return 0;
-    }
-
     Py_BEGIN_ALLOW_THREADS;
     pthread_mutex_lock(&conn->lock);
 
@@ -544,7 +578,7 @@ pq_abort(connectionObject *conn)
    connection without holding the global interpreter lock.
 */
 
-int
+RAISES_NEG int
 pq_reset_locked(connectionObject *conn, PGresult **pgres, char **error,
                 PyThreadState **tstate)
 {
@@ -560,12 +594,18 @@ pq_reset_locked(connectionObject *conn, PGresult **pgres, char **error,
         if (retvalue != 0) return retvalue;
     }
 
-    retvalue = pq_execute_command_locked(conn, "RESET ALL", pgres, error, tstate);
-    if (retvalue != 0) return retvalue;
+    if (conn->server_version >= 80300) {
+        retvalue = pq_execute_command_locked(conn, "DISCARD ALL", pgres, error, tstate);
+        if (retvalue != 0) return retvalue;
+    }
+    else {
+        retvalue = pq_execute_command_locked(conn, "RESET ALL", pgres, error, tstate);
+        if (retvalue != 0) return retvalue;
 
-    retvalue = pq_execute_command_locked(conn,
-        "SET SESSION AUTHORIZATION DEFAULT", pgres, error, tstate);
-    if (retvalue != 0) return retvalue;
+        retvalue = pq_execute_command_locked(conn,
+            "SET SESSION AUTHORIZATION DEFAULT", pgres, error, tstate);
+        if (retvalue != 0) return retvalue;
+    }
 
     /* should set the tpc xid to null: postponed until we get the GIL again */
     conn->status = CONN_STATUS_READY;
@@ -720,7 +760,7 @@ pq_tpc_command_locked(connectionObject *conn, const char *cmd, const char *tid,
     PyEval_RestoreThread(*tstate);
 
     /* convert the xid into the postgres transaction_id and quote it. */
-    if (!(etid = psycopg_escape_string((PyObject *)conn, tid, 0, NULL, NULL)))
+    if (!(etid = psycopg_escape_string(conn, tid, 0, NULL, NULL)))
     { goto exit; }
 
     /* prepare the command to the server */
@@ -751,7 +791,7 @@ exit:
    means that there is data available to be collected. -1 means an error, the
    exception will be set accordingly.
 
-   this fucntion locks the connection object
+   this function locks the connection object
    this function call Py_*_ALLOW_THREADS macros */
 
 int
@@ -767,6 +807,12 @@ pq_is_busy(connectionObject *conn)
         Dprintf("pq_is_busy: PQconsumeInput() failed");
         pthread_mutex_unlock(&(conn->lock));
         Py_BLOCK_THREADS;
+
+        /* if the libpq says pgconn is lost, close the py conn */
+        if (CONNECTION_BAD == PQstatus(conn->pgconn)) {
+            conn->closed = 2;
+        }
+
         PyErr_SetString(OperationalError, PQerrorMessage(conn->pgconn));
         return -1;
     }
@@ -796,6 +842,12 @@ pq_is_busy_locked(connectionObject *conn)
 
     if (PQconsumeInput(conn->pgconn) == 0) {
         Dprintf("pq_is_busy_locked: PQconsumeInput() failed");
+
+        /* if the libpq says pgconn is lost, close the py conn */
+        if (CONNECTION_BAD == PQstatus(conn->pgconn)) {
+            conn->closed = 2;
+        }
+
         PyErr_SetString(OperationalError, PQerrorMessage(conn->pgconn));
         return -1;
     }
@@ -832,12 +884,16 @@ pq_flush(connectionObject *conn)
 }
 
 /* pq_execute - execute a query, possibly asynchronously
+ *
+ * With no_result an eventual query result is discarded.
+ * Currently only used to implement cursor.executemany().
+ *
+ * This function locks the connection object
+ * This function call Py_*_ALLOW_THREADS macros
+*/
 
-   this fucntion locks the connection object
-   this function call Py_*_ALLOW_THREADS macros */
-
-int
-pq_execute(cursorObject *curs, const char *query, int async)
+RAISES_NEG int
+pq_execute(cursorObject *curs, const char *query, int async, int no_result)
 {
     PGresult *pgres = NULL;
     char *error = NULL;
@@ -846,8 +902,7 @@ pq_execute(cursorObject *curs, const char *query, int async)
     /* if the status of the connection is critical raise an exception and
        definitely close the connection */
     if (curs->conn->critical) {
-        pq_resolve_critical(curs->conn, 1);
-        return -1;
+        return pq_resolve_critical(curs->conn, 1);
     }
 
     /* check status of connection, raise error if not OK */
@@ -869,7 +924,7 @@ pq_execute(cursorObject *curs, const char *query, int async)
     }
 
     if (async == 0) {
-        IFCLEARPGRES(curs->pgres);
+        CLEARPGRES(curs->pgres);
         Dprintf("pq_execute: executing SYNC query: pgconn = %p", curs->conn->pgconn);
         Dprintf("    %-.200s", query);
         if (!psyco_green()) {
@@ -908,7 +963,7 @@ pq_execute(cursorObject *curs, const char *query, int async)
         Dprintf("pq_execute: executing ASYNC query: pgconn = %p", curs->conn->pgconn);
         Dprintf("    %-.200s", query);
 
-        IFCLEARPGRES(curs->pgres);
+        CLEARPGRES(curs->pgres);
         if (PQsendQuery(curs->conn->pgconn, query) == 0) {
             pthread_mutex_unlock(&(curs->conn->lock));
             Py_BLOCK_THREADS;
@@ -941,13 +996,14 @@ pq_execute(cursorObject *curs, const char *query, int async)
     /* if the execute was sync, we call pq_fetch() immediately,
        to respect the old DBAPI-2.0 compatible behaviour */
     if (async == 0) {
-        Dprintf("pq_execute: entering syncronous DBAPI compatibility mode");
-        if (pq_fetch(curs) == -1) return -1;
+        Dprintf("pq_execute: entering synchronous DBAPI compatibility mode");
+        if (pq_fetch(curs, no_result) < 0) return -1;
     }
     else {
+        PyObject *tmp;
         curs->conn->async_status = async_status;
-        curs->conn->async_cursor = PyWeakref_NewRef((PyObject *)curs, NULL);
-        if (!curs->conn->async_cursor) {
+        curs->conn->async_cursor = tmp = PyWeakref_NewRef((PyObject *)curs, NULL);
+        if (!tmp) {
             /* weakref creation failed */
             return -1;
         }
@@ -979,7 +1035,7 @@ pq_send_query(connectionObject *conn, const char *query)
 
 /* Return the last result available on the connection.
  *
- * The function will block will block only if a command is active and the
+ * The function will block only if a command is active and the
  * necessary response data has not yet been read by PQconsumeInput.
  *
  * The result should be disposed using PQclear()
@@ -1007,7 +1063,7 @@ pq_get_last_result(connectionObject *conn)
 
 /* pq_fetch - fetch data after a query
 
-   this fucntion locks the connection object
+   this function locks the connection object
    this function call Py_*_ALLOW_THREADS macros
 
    return value:
@@ -1016,7 +1072,7 @@ pq_get_last_result(connectionObject *conn)
       1 - result from backend (possibly data is ready)
 */
 
-static int
+RAISES_NEG static int
 _pq_fetch_tuples(cursorObject *curs)
 {
     int i, *dsize = NULL;
@@ -1100,9 +1156,8 @@ _pq_fetch_tuples(cursorObject *curs)
             cast = psyco_default_cast;
         }
 
-        Dprintf("_pq_fetch_tuples: using cast at %p (%s) for type %d",
-                cast, Bytes_AS_STRING(((typecastObject*)cast)->name),
-                PQftype(curs->pgres,i));
+        Dprintf("_pq_fetch_tuples: using cast at %p for type %d",
+                cast, PQftype(curs->pgres,i));
         Py_INCREF(cast);
         PyTuple_SET_ITEM(casts, i, cast);
 
@@ -1209,6 +1264,20 @@ exit:
     return rv;
 }
 
+void
+_read_rowcount(cursorObject *curs)
+{
+    const char *rowcount;
+
+    rowcount = PQcmdTuples(curs->pgres);
+    Dprintf("_read_rowcount: PQcmdTuples = %s", rowcount);
+    if (!rowcount || !rowcount[0]) {
+        curs->rowcount = -1;
+    } else {
+        curs->rowcount = atol(rowcount);
+    }
+}
+
 static int
 _pq_copy_in_v3(cursorObject *curs)
 {
@@ -1301,15 +1370,15 @@ _pq_copy_in_v3(cursorObject *curs)
     else if (error == 2)
         res = PQputCopyEnd(curs->conn->pgconn, "error in PQputCopyData() call");
     else
-        /* XXX would be nice to propagate the exeption */
+        /* XXX would be nice to propagate the exception */
         res = PQputCopyEnd(curs->conn->pgconn, "error in .read() call");
 
-    IFCLEARPGRES(curs->pgres);
-    
+    CLEARPGRES(curs->pgres);
+
     Dprintf("_pq_copy_in_v3: copy ended; res = %d", res);
-    
+
     /* if the result is -1 we should not even try to get a result from the
-       bacause that will lock the current thread forever */
+       because that will lock the current thread forever */
     if (res == -1) {
         pq_raise(curs->conn, curs, NULL);
         /* FIXME: pq_raise check the connection but for some reason even
@@ -1319,10 +1388,17 @@ _pq_copy_in_v3(cursorObject *curs)
     }
     else {
         /* and finally we grab the operation result from the backend */
-        while ((curs->pgres = PQgetResult(curs->conn->pgconn)) != NULL) {
+        for (;;) {
+            Py_BEGIN_ALLOW_THREADS;
+            curs->pgres = PQgetResult(curs->conn->pgconn);
+            Py_END_ALLOW_THREADS;
+
+            if (NULL == curs->pgres)
+                break;
+            _read_rowcount(curs);
             if (PQresultStatus(curs->pgres) == PGRES_FATAL_ERROR)
                 pq_raise(curs->conn, curs, NULL);
-            IFCLEARPGRES(curs->pgres);
+            CLEARPGRES(curs->pgres);
         }
     }
 
@@ -1388,11 +1464,18 @@ _pq_copy_out_v3(cursorObject *curs)
     }
 
     /* and finally we grab the operation result from the backend */
-    IFCLEARPGRES(curs->pgres);
-    while ((curs->pgres = PQgetResult(curs->conn->pgconn)) != NULL) {
+    CLEARPGRES(curs->pgres);
+    for (;;) {
+        Py_BEGIN_ALLOW_THREADS;
+        curs->pgres = PQgetResult(curs->conn->pgconn);
+        Py_END_ALLOW_THREADS;
+
+        if (NULL == curs->pgres)
+            break;
+        _read_rowcount(curs);
         if (PQresultStatus(curs->pgres) == PGRES_FATAL_ERROR)
             pq_raise(curs->conn, curs, NULL);
-        IFCLEARPGRES(curs->pgres);
+        CLEARPGRES(curs->pgres);
     }
     ret = 1;
 
@@ -1402,10 +1485,9 @@ exit:
 }
 
 int
-pq_fetch(cursorObject *curs)
+pq_fetch(cursorObject *curs, int no_result)
 {
     int pgstatus, ex = -1;
-    const char *rowcount;
 
     /* even if we fail, we remove any information about the previous query */
     curs_reset(curs);
@@ -1416,8 +1498,7 @@ pq_fetch(cursorObject *curs)
         Dprintf("pq_fetch: got a NULL pgres, checking for critical");
         pq_set_critical(curs->conn);
         if (curs->conn->critical) {
-            pq_resolve_critical(curs->conn);
-            return -1;
+            return pq_resolve_critical(curs->conn);
         }
         else {
             return 0;
@@ -1438,11 +1519,7 @@ pq_fetch(cursorObject *curs)
 
     case PGRES_COMMAND_OK:
         Dprintf("pq_fetch: command returned OK (no tuples)");
-        rowcount = PQcmdTuples(curs->pgres);
-        if (!rowcount || !rowcount[0])
-          curs->rowcount = -1;
-        else
-          curs->rowcount = atoi(rowcount);
+        _read_rowcount(curs);
         curs->lastoid = PQoidValue(curs->pgres);
         CLEARPGRES(curs->pgres);
         ex = 1;
@@ -1450,40 +1527,47 @@ pq_fetch(cursorObject *curs)
 
     case PGRES_COPY_OUT:
         Dprintf("pq_fetch: data from a COPY TO (no tuples)");
-        ex = _pq_copy_out_v3(curs);
         curs->rowcount = -1;
+        ex = _pq_copy_out_v3(curs);
         /* error caught by out glorious notice handler */
         if (PyErr_Occurred()) ex = -1;
-        IFCLEARPGRES(curs->pgres);
+        CLEARPGRES(curs->pgres);
         break;
 
     case PGRES_COPY_IN:
         Dprintf("pq_fetch: data from a COPY FROM (no tuples)");
-        ex = _pq_copy_in_v3(curs);
         curs->rowcount = -1;
+        ex = _pq_copy_in_v3(curs);
         /* error caught by out glorious notice handler */
         if (PyErr_Occurred()) ex = -1;
-        IFCLEARPGRES(curs->pgres);
+        CLEARPGRES(curs->pgres);
         break;
 
     case PGRES_TUPLES_OK:
-        Dprintf("pq_fetch: data from a SELECT (got tuples)");
-        curs->rowcount = PQntuples(curs->pgres);
-        if (0 == _pq_fetch_tuples(curs)) { ex = 0; }
-        /* don't clear curs->pgres, because it contains the results! */
+        if (!no_result) {
+            Dprintf("pq_fetch: got tuples");
+            curs->rowcount = PQntuples(curs->pgres);
+            if (0 == _pq_fetch_tuples(curs)) { ex = 0; }
+            /* don't clear curs->pgres, because it contains the results! */
+        }
+        else {
+            Dprintf("pq_fetch: got tuples, discarding them");
+            CLEARPGRES(curs->pgres);
+            curs->rowcount = -1;
+            ex = 0;
+        }
         break;
 
     case PGRES_EMPTY_QUERY:
         PyErr_SetString(ProgrammingError,
             "can't execute an empty query");
-        IFCLEARPGRES(curs->pgres);
+        CLEARPGRES(curs->pgres);
         ex = -1;
         break;
 
     default:
         Dprintf("pq_fetch: uh-oh, something FAILED: pgconn = %p", curs->conn);
         pq_raise(curs->conn, curs, NULL);
-        IFCLEARPGRES(curs->pgres);
         ex = -1;
         break;
     }
@@ -1493,13 +1577,7 @@ pq_fetch(cursorObject *curs)
        raise the exception but we avoid to close the connection) */
     Dprintf("pq_fetch: fetching done; check for critical errors");
     if (curs->conn->critical) {
-        if (ex == -1) {
-            pq_resolve_critical(curs->conn, 1);
-        }
-        else {
-            pq_resolve_critical(curs->conn, 0);
-        }
-        return -1;
+        return pq_resolve_critical(curs->conn, ex == -1 ? 1 : 0);
     }
 
     return ex;
