@@ -35,6 +35,8 @@
 #include "psycopg/typecast.h"
 #include "psycopg/microprotocols.h"
 #include "psycopg/microprotocols_proto.h"
+#include "psycopg/error.h"
+#include "psycopg/diagnostics.h"
 
 #include "psycopg/adapter_qstring.h"
 #include "psycopg/adapter_binary.h"
@@ -143,14 +145,6 @@ psyco_connect(PyObject *self, PyObject *args, PyObject *keywds)
 "  * `name`: Name for the new type\n" \
 "  * `baseobj`: Adapter to perform type conversion of a single array item."
 
-static void
-_psyco_register_type_set(PyObject **dict, PyObject *type)
-{
-    if (*dict == NULL)
-        *dict = PyDict_New();
-    typecast_add(type, *dict, 0);
-}
-
 static PyObject *
 psyco_register_type(PyObject *self, PyObject *args)
 {
@@ -162,10 +156,16 @@ psyco_register_type(PyObject *self, PyObject *args)
 
     if (obj != NULL && obj != Py_None) {
         if (PyObject_TypeCheck(obj, &cursorType)) {
-            _psyco_register_type_set(&(((cursorObject*)obj)->string_types), type);
+            PyObject **dict = &(((cursorObject*)obj)->string_types);
+            if (*dict == NULL) {
+                if (!(*dict = PyDict_New())) { return NULL; }
+            }
+            if (0 > typecast_add(type, *dict, 0)) { return NULL; }
         }
         else if (PyObject_TypeCheck(obj, &connectionType)) {
-            typecast_add(type, ((connectionObject*)obj)->string_types, 0);
+            if (0 > typecast_add(type, ((connectionObject*)obj)->string_types, 0)) {
+                return NULL;
+            }
         }
         else {
             PyErr_SetString(PyExc_TypeError,
@@ -174,74 +174,115 @@ psyco_register_type(PyObject *self, PyObject *args)
         }
     }
     else {
-        typecast_add(type, NULL, 0);
+        if (0 > typecast_add(type, NULL, 0)) { return NULL; }
     }
 
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 
-/* default adapters */
-
-static void
+/* Initialize the default adapters map
+ *
+ * Return 0 on success, else -1 and set an exception.
+ */
+static int
 psyco_adapters_init(PyObject *mod)
 {
-    PyObject *call;
+    PyObject *call = NULL;
+    int rv = -1;
 
-    microprotocols_add(&PyFloat_Type, NULL, (PyObject*)&pfloatType);
+    if (0 != microprotocols_add(&PyFloat_Type, NULL, (PyObject*)&pfloatType)) {
+        goto exit;
+    }
 #if PY_MAJOR_VERSION < 3
-    microprotocols_add(&PyInt_Type, NULL, (PyObject*)&pintType);
+    if (0 != microprotocols_add(&PyInt_Type, NULL, (PyObject*)&pintType)) {
+        goto exit;
+    }
 #endif
-    microprotocols_add(&PyLong_Type, NULL, (PyObject*)&pintType);
-    microprotocols_add(&PyBool_Type, NULL, (PyObject*)&pbooleanType);
+    if (0 != microprotocols_add(&PyLong_Type, NULL, (PyObject*)&pintType)) {
+        goto exit;
+    }
+    if (0 != microprotocols_add(&PyBool_Type, NULL, (PyObject*)&pbooleanType)) {
+        goto exit;
+    }
 
     /* strings */
 #if PY_MAJOR_VERSION < 3
-    microprotocols_add(&PyString_Type, NULL, (PyObject*)&qstringType);
+    if (0 != microprotocols_add(&PyString_Type, NULL, (PyObject*)&qstringType)) {
+        goto exit;
+    }
 #endif
-    microprotocols_add(&PyUnicode_Type, NULL, (PyObject*)&qstringType);
+    if (0 != microprotocols_add(&PyUnicode_Type, NULL, (PyObject*)&qstringType)) {
+        goto exit;
+    }
 
     /* binary */
 #if PY_MAJOR_VERSION < 3
-    microprotocols_add(&PyBuffer_Type, NULL, (PyObject*)&binaryType);
+    if (0 != microprotocols_add(&PyBuffer_Type, NULL, (PyObject*)&binaryType)) {
+        goto exit;
+    }
 #else
-    microprotocols_add(&PyBytes_Type, NULL, (PyObject*)&binaryType);
+    if (0 != microprotocols_add(&PyBytes_Type, NULL, (PyObject*)&binaryType)) {
+        goto exit;
+    }
 #endif
 
 #if PY_MAJOR_VERSION >= 3 || PY_MINOR_VERSION >= 6
-    microprotocols_add(&PyByteArray_Type, NULL, (PyObject*)&binaryType);
+    if (0 != microprotocols_add(&PyByteArray_Type, NULL, (PyObject*)&binaryType)) {
+        goto exit;
+    }
 #endif
 #if PY_MAJOR_VERSION >= 3 || PY_MINOR_VERSION >= 7
-    microprotocols_add(&PyMemoryView_Type, NULL, (PyObject*)&binaryType);
+    if (0 != microprotocols_add(&PyMemoryView_Type, NULL, (PyObject*)&binaryType)) {
+        goto exit;
+    }
 #endif
 
-    microprotocols_add(&PyList_Type, NULL, (PyObject*)&listType);
+    if (0 != microprotocols_add(&PyList_Type, NULL, (PyObject*)&listType)) {
+        goto exit;
+    }
 
     /* the module has already been initialized, so we can obtain the callable
        objects directly from its dictionary :) */
-    call = PyMapping_GetItemString(mod, "DateFromPy");
-    microprotocols_add(PyDateTimeAPI->DateType, NULL, call);
-    call = PyMapping_GetItemString(mod, "TimeFromPy");
-    microprotocols_add(PyDateTimeAPI->TimeType, NULL, call);
-    call = PyMapping_GetItemString(mod, "TimestampFromPy");
-    microprotocols_add(PyDateTimeAPI->DateTimeType, NULL, call);
-    call = PyMapping_GetItemString(mod, "IntervalFromPy");
-    microprotocols_add(PyDateTimeAPI->DeltaType, NULL, call);
+    if (!(call = PyMapping_GetItemString(mod, "DateFromPy"))) { goto exit; }
+    if (0 != microprotocols_add(PyDateTimeAPI->DateType, NULL, call)) { goto exit; }
+    Py_CLEAR(call);
+
+    if (!(call = PyMapping_GetItemString(mod, "TimeFromPy"))) { goto exit; }
+    if (0 != microprotocols_add(PyDateTimeAPI->TimeType, NULL, call)) { goto exit; }
+    Py_CLEAR(call);
+
+    if (!(call = PyMapping_GetItemString(mod, "TimestampFromPy"))) { goto exit; }
+    if (0 != microprotocols_add(PyDateTimeAPI->DateTimeType, NULL, call)) { goto exit; }
+    Py_CLEAR(call);
+
+    if (!(call = PyMapping_GetItemString(mod, "IntervalFromPy"))) { goto exit; }
+    if (0 != microprotocols_add(PyDateTimeAPI->DeltaType, NULL, call)) { goto exit; }
+    Py_CLEAR(call);
 
 #ifdef HAVE_MXDATETIME
     /* as above, we use the callable objects from the psycopg module */
     if (NULL != (call = PyMapping_GetItemString(mod, "TimestampFromMx"))) {
-        microprotocols_add(mxDateTime.DateTime_Type, NULL, call);
+        if (0 != microprotocols_add(mxDateTime.DateTime_Type, NULL, call)) { goto exit; }
+        Py_CLEAR(call);
 
         /* if we found the above, we have this too. */
-        call = PyMapping_GetItemString(mod, "TimeFromMx");
-        microprotocols_add(mxDateTime.DateTimeDelta_Type, NULL, call);
+        if (!(call = PyMapping_GetItemString(mod, "TimeFromMx"))) { goto exit; }
+        if (0 != microprotocols_add(mxDateTime.DateTimeDelta_Type, NULL, call)) { goto exit; }
+        Py_CLEAR(call);
     }
     else {
         PyErr_Clear();
     }
 #endif
+
+    /* Success! */
+    rv = 0;
+
+exit:
+    Py_XDECREF(call);
+
+    return rv;
 }
 
 /* psyco_encodings_fill
@@ -326,15 +367,27 @@ static encodingPair encodings[] = {
     {NULL, NULL}
 };
 
-static void psyco_encodings_fill(PyObject *dict)
+/* Initialize the encodings table.
+ *
+ * Return 0 on success, else -1 and set an exception.
+ */
+static int psyco_encodings_fill(PyObject *dict)
 {
+    PyObject *value = NULL;
     encodingPair *enc;
+    int rv = -1;
 
     for (enc = encodings; enc->pgenc != NULL; enc++) {
-        PyObject *value = Text_FromUTF8(enc->pyenc);
-        PyDict_SetItemString(dict, enc->pgenc, value);
-        Py_DECREF(value);
+        if (!(value = Text_FromUTF8(enc->pyenc))) { goto exit; }
+        if (0 != PyDict_SetItemString(dict, enc->pgenc, value)) { goto exit; }
+        Py_CLEAR(value);
     }
+    rv = 0;
+
+exit:
+    Py_XDECREF(value);
+
+    return rv;
 }
 
 /* psyco_errors_init, psyco_errors_fill (callable from C)
@@ -356,8 +409,8 @@ static struct {
     PyObject **base;
     const char *docstr;
 } exctable[] = {
-    { "psycopg2.Error", &Error, 0, Error_doc },
-    { "psycopg2.Warning", &Warning, 0, Warning_doc },
+    { "psycopg2.Error", &Error, NULL, Error_doc },
+    { "psycopg2.Warning", &Warning, NULL, Warning_doc },
     { "psycopg2.InterfaceError", &InterfaceError, &Error, InterfaceError_doc },
     { "psycopg2.DatabaseError", &DatabaseError, &Error, DatabaseError_doc },
     { "psycopg2.InternalError", &InternalError, &DatabaseError, InternalError_doc },
@@ -380,97 +433,92 @@ static struct {
     {NULL}  /* Sentinel */
 };
 
-static void
+
+static int
 psyco_errors_init(void)
 {
-    /* the names of the exceptions here reflect the oranization of the
+    /* the names of the exceptions here reflect the organization of the
        psycopg2 module and not the fact the the original error objects
        live in _psycopg */
 
     int i;
-    PyObject *dict;
-    PyObject *base;
-    PyObject *str;
+    PyObject *dict = NULL;
+    PyObject *str = NULL;
+    int rv = -1;
 
-    for (i=0; exctable[i].name; i++) {
-        dict = PyDict_New();
+    /* 'Error' has been defined elsewhere: only init the other classes */
+    Error = (PyObject *)&errorType;
+
+    for (i = 1; exctable[i].name; i++) {
+        if (!(dict = PyDict_New())) { goto exit; }
 
         if (exctable[i].docstr) {
-            str = Text_FromUTF8(exctable[i].docstr);
-            PyDict_SetItemString(dict, "__doc__", str);
+            if (!(str = Text_FromUTF8(exctable[i].docstr))) { goto exit; }
+            if (0 != PyDict_SetItemString(dict, "__doc__", str)) { goto exit; }
+            Py_CLEAR(str);
         }
 
-        if (exctable[i].base == 0) {
-            #if PY_MAJOR_VERSION < 3
-            base = PyExc_StandardError;
-            #else
-            /* StandardError is gone in 3.0 */
-            base = NULL;
-            #endif
+        /* can't put PyExc_StandardError in the static exctable:
+         * windows build will fail */
+        if (!(*exctable[i].exc = PyErr_NewException(
+                exctable[i].name,
+                exctable[i].base ? *exctable[i].base : PyExc_StandardError,
+                dict))) {
+            goto exit;
         }
-        else
-            base = *exctable[i].base;
-
-        *exctable[i].exc = PyErr_NewException(exctable[i].name, base, dict);
+        Py_CLEAR(dict);
     }
 
-    /* Make pgerror, pgcode and cursor default to None on psycopg
-       error objects.  This simplifies error handling code that checks
-       these attributes. */
-    PyObject_SetAttrString(Error, "pgerror", Py_None);
-    PyObject_SetAttrString(Error, "pgcode", Py_None);
-    PyObject_SetAttrString(Error, "cursor", Py_None);
+    rv = 0;
+
+exit:
+    Py_XDECREF(str);
+    Py_XDECREF(dict);
+    return rv;
 }
 
 void
 psyco_errors_fill(PyObject *dict)
 {
-    PyDict_SetItemString(dict, "Error", Error);
-    PyDict_SetItemString(dict, "Warning", Warning);
-    PyDict_SetItemString(dict, "InterfaceError", InterfaceError);
-    PyDict_SetItemString(dict, "DatabaseError", DatabaseError);
-    PyDict_SetItemString(dict, "InternalError", InternalError);
-    PyDict_SetItemString(dict, "OperationalError", OperationalError);
-    PyDict_SetItemString(dict, "ProgrammingError", ProgrammingError);
-    PyDict_SetItemString(dict, "IntegrityError", IntegrityError);
-    PyDict_SetItemString(dict, "DataError", DataError);
-    PyDict_SetItemString(dict, "NotSupportedError", NotSupportedError);
-#ifdef PSYCOPG_EXTENSIONS
-    PyDict_SetItemString(dict, "QueryCanceledError", QueryCanceledError);
-    PyDict_SetItemString(dict, "TransactionRollbackError",
-                         TransactionRollbackError);
-#endif
+    int i;
+    char *name;
+
+    for (i = 0; exctable[i].name; i++) {
+        if (NULL == exctable[i].exc) { continue; }
+
+        /* the name is the part after the last dot */
+        name = strrchr(exctable[i].name, '.');
+        name = name ? name + 1 : exctable[i].name;
+
+        PyDict_SetItemString(dict, name, *exctable[i].exc);
+    }
 }
 
 void
 psyco_errors_set(PyObject *type)
 {
-    PyObject_SetAttrString(type, "Error", Error);
-    PyObject_SetAttrString(type, "Warning", Warning);
-    PyObject_SetAttrString(type, "InterfaceError", InterfaceError);
-    PyObject_SetAttrString(type, "DatabaseError", DatabaseError);
-    PyObject_SetAttrString(type, "InternalError", InternalError);
-    PyObject_SetAttrString(type, "OperationalError", OperationalError);
-    PyObject_SetAttrString(type, "ProgrammingError", ProgrammingError);
-    PyObject_SetAttrString(type, "IntegrityError", IntegrityError);
-    PyObject_SetAttrString(type, "DataError", DataError);
-    PyObject_SetAttrString(type, "NotSupportedError", NotSupportedError);
-#ifdef PSYCOPG_EXTENSIONS
-    PyObject_SetAttrString(type, "QueryCanceledError", QueryCanceledError);
-    PyObject_SetAttrString(type, "TransactionRollbackError",
-                           TransactionRollbackError);
-#endif
+    int i;
+    char *name;
+
+    for (i = 0; exctable[i].name; i++) {
+        if (NULL == exctable[i].exc) { continue; }
+
+        /* the name is the part after the last dot */
+        name = strrchr(exctable[i].name, '.');
+        name = name ? name + 1 : exctable[i].name;
+
+        PyObject_SetAttrString(type, name, *exctable[i].exc);
+    }
 }
 
-/* psyco_error_new
+/* psyco_set_error
 
    Create a new error of the given type with extra attributes. */
 
-void
-psyco_set_error(PyObject *exc, cursorObject *curs, const char *msg,
-                const char *pgerror, const char *pgcode)
+/* TODO: may have been changed to BORROWED */
+RAISES PyObject *
+psyco_set_error(PyObject *exc, cursorObject *curs, const char *msg)
 {
-    PyObject *t;
     PyObject *pymsg;
     PyObject *err = NULL;
     connectionObject *conn = NULL;
@@ -486,29 +534,24 @@ psyco_set_error(PyObject *exc, cursorObject *curs, const char *msg,
     else {
         /* what's better than an error in an error handler in the morning?
          * Anyway, some error was set, refcount is ok... get outta here. */
-        return;
+        return NULL;
+    }
+
+    if (err && PyObject_TypeCheck(err, &errorType)) {
+        errorObject *perr = (errorObject *)err;
+        if (curs) {
+            Py_CLEAR(perr->cursor);
+            Py_INCREF(curs);
+            perr->cursor = curs;
+        }
     }
 
     if (err) {
-        if (curs) {
-            PyObject_SetAttrString(err, "cursor", (PyObject *)curs);
-        }
-
-        if (pgerror) {
-            t = conn_text_from_chars(conn, pgerror);
-            PyObject_SetAttrString(err, "pgerror", t);
-            Py_DECREF(t);
-        }
-
-        if (pgcode) {
-            t = conn_text_from_chars(conn, pgcode);
-            PyObject_SetAttrString(err, "pgcode", t);
-            Py_DECREF(t);
-        }
-
         PyErr_SetObject(exc, err);
         Py_DECREF(err);
     }
+
+    return err;
 }
 
 
@@ -570,7 +613,7 @@ psyco_GetDecimalType(void)
     }
 
     /* Store the object from future uses. */
-    if (can_cache && !cachedType) {
+    if (can_cache && !cachedType && decimalType) {
         Py_INCREF(decimalType);
         cachedType = decimalType;
     }
@@ -594,15 +637,11 @@ psyco_make_description_type(void)
     /* Try to import collections.namedtuple */
     if (!(coll = PyImport_ImportModule("collections"))) {
         Dprintf("psyco_make_description_type: collections import failed");
-        PyErr_Clear();
-        rv = Py_None;
-        goto exit;
+        goto error;
     }
     if (!(nt = PyObject_GetAttrString(coll, "namedtuple"))) {
         Dprintf("psyco_make_description_type: no collections.namedtuple");
-        PyErr_Clear();
-        rv = Py_None;
-        goto exit;
+        goto error;
     }
 
     /* Build the namedtuple */
@@ -614,6 +653,13 @@ exit:
     Py_XDECREF(nt);
 
     return rv;
+
+error:
+    /* controlled error: we will fall back to regular tuples. Return None. */
+    PyErr_Clear();
+    rv = Py_None;
+    Py_INCREF(rv);
+    goto exit;
 }
 
 
@@ -725,39 +771,59 @@ INIT_MODULE(_psycopg)(void)
 
     /* initialize all the new types and then the module */
     Py_TYPE(&connectionType) = &PyType_Type;
-    Py_TYPE(&cursorType)     = &PyType_Type;
-    Py_TYPE(&typecastType)   = &PyType_Type;
-    Py_TYPE(&qstringType)    = &PyType_Type;
-    Py_TYPE(&binaryType)     = &PyType_Type;
-    Py_TYPE(&isqlquoteType)  = &PyType_Type;
-    Py_TYPE(&pbooleanType)   = &PyType_Type;
-    Py_TYPE(&pintType)       = &PyType_Type;
-    Py_TYPE(&pfloatType)     = &PyType_Type;
-    Py_TYPE(&pdecimalType)   = &PyType_Type;
-    Py_TYPE(&asisType)       = &PyType_Type;
-    Py_TYPE(&listType)       = &PyType_Type;
-    Py_TYPE(&chunkType)      = &PyType_Type;
-    Py_TYPE(&NotifyType)     = &PyType_Type;
-    Py_TYPE(&XidType)        = &PyType_Type;
-
     if (PyType_Ready(&connectionType) == -1) goto exit;
+
+    Py_TYPE(&cursorType) = &PyType_Type;
     if (PyType_Ready(&cursorType) == -1) goto exit;
+
+    Py_TYPE(&typecastType) = &PyType_Type;
     if (PyType_Ready(&typecastType) == -1) goto exit;
+
+    Py_TYPE(&qstringType) = &PyType_Type;
     if (PyType_Ready(&qstringType) == -1) goto exit;
+
+    Py_TYPE(&binaryType) = &PyType_Type;
     if (PyType_Ready(&binaryType) == -1) goto exit;
+
+    Py_TYPE(&isqlquoteType) = &PyType_Type;
     if (PyType_Ready(&isqlquoteType) == -1) goto exit;
+
+    Py_TYPE(&pbooleanType) = &PyType_Type;
     if (PyType_Ready(&pbooleanType) == -1) goto exit;
+
+    Py_TYPE(&pintType) = &PyType_Type;
     if (PyType_Ready(&pintType) == -1) goto exit;
+
+    Py_TYPE(&pfloatType) = &PyType_Type;
     if (PyType_Ready(&pfloatType) == -1) goto exit;
+
+    Py_TYPE(&pdecimalType) = &PyType_Type;
     if (PyType_Ready(&pdecimalType) == -1) goto exit;
+
+    Py_TYPE(&asisType) = &PyType_Type;
     if (PyType_Ready(&asisType) == -1) goto exit;
+
+    Py_TYPE(&listType) = &PyType_Type;
     if (PyType_Ready(&listType) == -1) goto exit;
+
+    Py_TYPE(&chunkType) = &PyType_Type;
     if (PyType_Ready(&chunkType) == -1) goto exit;
-    if (PyType_Ready(&NotifyType) == -1) goto exit;
-    if (PyType_Ready(&XidType) == -1) goto exit;
+
+    Py_TYPE(&notifyType) = &PyType_Type;
+    if (PyType_Ready(&notifyType) == -1) goto exit;
+
+    Py_TYPE(&xidType) = &PyType_Type;
+    if (PyType_Ready(&xidType) == -1) goto exit;
+
+    Py_TYPE(&errorType) = &PyType_Type;
+    errorType.tp_base = (PyTypeObject *)PyExc_StandardError;
+    if (PyType_Ready(&errorType) == -1) goto exit;
+
+    Py_TYPE(&diagnosticsType) = &PyType_Type;
+    if (PyType_Ready(&diagnosticsType) == -1) goto exit;
 
 #ifdef PSYCOPG_EXTENSIONS
-    Py_TYPE(&lobjectType)    = &PyType_Type;
+    Py_TYPE(&lobjectType) = &PyType_Type;
     if (PyType_Ready(&lobjectType) == -1) goto exit;
 #endif
 
@@ -765,6 +831,7 @@ INIT_MODULE(_psycopg)(void)
 #ifdef HAVE_MXDATETIME
     Py_TYPE(&mxdatetimeType) = &PyType_Type;
     if (PyType_Ready(&mxdatetimeType) == -1) goto exit;
+
     if (0 != mxDateTime_ImportModuleAndAPI()) {
         PyErr_Clear();
 
@@ -826,10 +893,10 @@ INIT_MODULE(_psycopg)(void)
 #endif
 
     /* other mixed initializations of module-level variables */
-    psycoEncodings = PyDict_New();
-    psyco_encodings_fill(psycoEncodings);
+    if (!(psycoEncodings = PyDict_New())) { goto exit; }
+    if (0 != psyco_encodings_fill(psycoEncodings)) { goto exit; }
     psyco_null = Bytes_FromString("NULL");
-    psyco_DescriptionType = psyco_make_description_type();
+    if (!(psyco_DescriptionType = psyco_make_description_type())) { goto exit; }
 
     /* set some module's parameters */
     PyModule_AddStringConstant(module, "__version__", PSYCOPG_VERSION);
@@ -842,8 +909,9 @@ INIT_MODULE(_psycopg)(void)
     PyModule_AddObject(module, "connection", (PyObject*)&connectionType);
     PyModule_AddObject(module, "cursor", (PyObject*)&cursorType);
     PyModule_AddObject(module, "ISQLQuote", (PyObject*)&isqlquoteType);
-    PyModule_AddObject(module, "Notify", (PyObject*)&NotifyType);
-    PyModule_AddObject(module, "Xid", (PyObject*)&XidType);
+    PyModule_AddObject(module, "Notify", (PyObject*)&notifyType);
+    PyModule_AddObject(module, "Xid", (PyObject*)&xidType);
+    PyModule_AddObject(module, "Diagnostics", (PyObject*)&diagnosticsType);
 #ifdef PSYCOPG_EXTENSIONS
     PyModule_AddObject(module, "lobject", (PyObject*)&lobjectType);
 #endif
@@ -862,41 +930,16 @@ INIT_MODULE(_psycopg)(void)
     }
 #endif
     /* initialize default set of typecasters */
-    typecast_init(dict);
+    if (0 != typecast_init(dict)) { goto exit; }
 
     /* initialize microprotocols layer */
     microprotocols_init(dict);
-    psyco_adapters_init(dict);
+    if (0 != psyco_adapters_init(dict)) { goto exit; }
 
     /* create a standard set of exceptions and add them to the module's dict */
-    psyco_errors_init();
+    if (0 != psyco_errors_init()) { goto exit; }
     psyco_errors_fill(dict);
 
-    /* Solve win32 build issue about non-constant initializer element */
-    cursorType.tp_alloc = PyType_GenericAlloc;
-    binaryType.tp_alloc = PyType_GenericAlloc;
-    isqlquoteType.tp_alloc = PyType_GenericAlloc;
-    pbooleanType.tp_alloc = PyType_GenericAlloc;
-    pintType.tp_alloc = PyType_GenericAlloc;
-    pfloatType.tp_alloc = PyType_GenericAlloc;
-    pdecimalType.tp_alloc = PyType_GenericAlloc;
-    connectionType.tp_alloc = PyType_GenericAlloc;
-    asisType.tp_alloc = PyType_GenericAlloc;
-    qstringType.tp_alloc = PyType_GenericAlloc;
-    listType.tp_alloc = PyType_GenericAlloc;
-    chunkType.tp_alloc = PyType_GenericAlloc;
-    pydatetimeType.tp_alloc = PyType_GenericAlloc;
-    NotifyType.tp_alloc = PyType_GenericAlloc;
-    XidType.tp_alloc = PyType_GenericAlloc;
-
-#ifdef PSYCOPG_EXTENSIONS
-    lobjectType.tp_alloc = PyType_GenericAlloc;
-#endif
-
-#ifdef HAVE_MXDATETIME
-    mxdatetimeType.tp_alloc = PyType_GenericAlloc;
-#endif
-
     Dprintf("initpsycopg: module initialization complete");
 
 exit:
